/*!
 * @file fi_durations.h
 * @brief Statistical support object for tracking duration intervals.
 *
 * Copyright 2020 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FI_DURATION_H_
#define FI_DURATION_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------
#include <chrono>

#include "fas_video.h"

namespace FAS_VIDEO
{

//--------------------------------------------------------------------
// Constants, Typedefs and Macros
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declarations
//--------------------------------------------------------------------

//! The FiDurations Class is utility class to track time in milliseconds
//! since construction.  Used for statistic collection.
class FiDurations
{
public:
    FiDurations();

    virtual ~FiDurations();

    //! This object is non-copyable and non-copy assignable.
    FiDurations(const FiDurations& i) = delete;
    FiDurations& operator=(const FiDurations& i) = delete;

    void MarkStart();
    void MarkInterval();
    static void LogDurationSeparator();
    void LogDurationInfo(const char *logInfoStr);

    uint32_t GetTotalIntervalTimeInMS();

    std::string DurationToTimeStr(uint32_t msTimeInMS);

protected:
    uint32_t numberOfIntervals;
    uint32_t averageIntervalInMs;
    uint32_t smallestIntervalInMS;
    uint32_t biggestIntervalInMS;
    uint32_t totalIntervalTimeInMS;
    std::chrono::system_clock::time_point startTime;
    std::chrono::system_clock::time_point constructTime;
};

} /* End of namespace FAS_VIDEO */

#endif /* !FI_DURATION_H_ */
