/*!
 * @file fi_jpeg.h
 * @brief Definitions and declarations for Fastec JPEG images.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FIVID_FI_JPEG_
#define FIVID_FI_JPEG_

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <memory>
#include <vector>

#include "fas_video.h"
#include "fi_image.h"

#include <stdint.h>

namespace FI_JPEG
{
//--------------------------------------------------------------------
// Constant, Typedefs and Macros
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class FiJPEG : public FAS_VIDEO::FiImage
{
public:
    FiJPEG();
    virtual ~FiJPEG() {}

    virtual int CreateHeader(uint32_t fourcc, const pm_metadata *meta) override;
    std::unique_ptr<FAS_VIDEO::HEADER_DATA> GetHeader() override;

    //! @name Public Property Accessor Functions.
    //!
    //! These are the public functions used to access the properties of this object.
    //! @{

    virtual FAS_VIDEO::IMAGE_FORMAT GetImageFormat() override;
    virtual uint32_t GetImageWidth() override;
    virtual uint32_t GetImageHeight() override;
    virtual uint32_t GetBitsPerPixel() override;
    virtual uint32_t GetRowAlign() override;
    virtual uint32_t GetFullFramesize() override;

    //! @}

protected:
    virtual bool VerifyImageData(const uint8_t *imageDataToVerify, uint32_t imageDataSize) override;

private:
    uint32_t attachedImageWidth;    // The image height as interpreted from Start Of Frame Zer0 tag (SOF0).
    uint32_t attachedImageHeight;   // The image width as interpreted from Start Of Frame Zer0 tag (SOF0).
    uint32_t attachedComponentSize; // Should be 1 (Y only) or 3 (YCrCb) or 3(RGB) as interpreted from (SOF0).
    uint32_t attachedPrecision;     // Should be 8 bits per pixel per color component from (SOF0).
};

} // End of FI_JPEG

#endif  /* FIVID_FI_JPEG_ */

